@@ -2,45 +2,52 @@
 # crypto and anonymity library.
 #
 # Licensed under the AGPLv3: https://www.gnu.org/licenses/agpl-3.0.html
-# Copyright © 2019-2021 Gonzo Investigatory Journalism Agency, LLC
+# Copyright © 2019-2021 Gonzo Investigative Journalism Agency, LLC
 #            <gonzo.development@protonmail.ch>
 #           © 2019-2021 Richard Machado <rmlibre@riseup.net>
 # All rights reserved.
 #
 
 
-__all__ = []
+__all__ = [
+    "pad_plaintext",
+    "apad_plaintext",
+    "depad_plaintext",
+    "adepad_plaintext",
+]
 
 
-__doc__ = """
-Coordinates some of the library's UI/UX by inserting higher-level
-functionality from dependant modules into lower-level modules in this
-package.
-"""
+__doc__ = (
+    "Coordinates some of the library's UI/UX by inserting higher-level "
+    "functionality from dependant modules into lower-level modules in "
+    "this package."
+)
 
 
+from collections import deque
 from .debuggers import gen_timer, agen_timer
 from .generics import azip
+from .generics import Padding
+from .generics import Datastream
+from .generics import data, adata
 from .generics import sha_512, asha_512
 from .generics import Comprende, comprehension
 from .generics import convert_static_method_to_member
 from .randoms import random_sleep as _random_sleep
 from .randoms import arandom_sleep as _arandom_sleep
 from .ciphers import csprng
-from .ciphers import Ropake
-from .ciphers import X25519
-from .ciphers import validator
 from .ciphers import Passcrypt
-from .ciphers import OneTimePad
+from .ciphers import Chunky2048
 from .ciphers import passcrypt as _passcrypt
 from .ciphers import apasscrypt as _apasscrypt
 from .ciphers import generate_salt, agenerate_salt
+from .keygens import Ropake
+from .keygens import X25519
 from .keygens import Keys, AsyncKeys
-from .keygens import insert_keyrings
+from .keygens import insert_keygens
 
 
-
-@comprehension()
+@comprehension(chained=True)
 async def adebugger(self, *args, **kwargs):
     """
     Allows users to benchmark & read inspection details of running async
@@ -52,7 +59,7 @@ async def adebugger(self, *args, **kwargs):
         yield result
 
 
-@comprehension()
+@comprehension(chained=True)
 def debugger(self, *args, **kwargs):
     """
     Allows users to benchmark & read inspection details of running sync
@@ -64,7 +71,7 @@ def debugger(self, *args, **kwargs):
         yield result
 
 
-@comprehension()
+@comprehension(chained=True)
 async def abytes_xor(self, *, key, validator):
     """
     Applies an xor to each result of any underlying async `Comprende`
@@ -72,12 +79,12 @@ async def abytes_xor(self, *, key, validator):
     The underlying ``self`` async generator needs to produce 256-byte
     integers to be xor'd on each iteration.
     """
-    xoring = OneTimePad.abytes_xor.root(self, key=key, validator=validator)
+    xoring = Chunky2048.abytes_xor.root(self, key=key, validator=validator)
     async for result in xoring:
         yield result
 
 
-@comprehension()
+@comprehension(chained=True)
 def bytes_xor(self, *, key, validator):
     """
     Applies an xor to each result of any underlying sync `Comprende`
@@ -85,12 +92,12 @@ def bytes_xor(self, *, key, validator):
     underlying ``self`` sync generator needs to produce 256-byte
     integers to be xor'd on each iteration.
     """
-    xoring = OneTimePad.bytes_xor.root(self, key=key, validator=validator)
+    xoring = Chunky2048.bytes_xor.root(self, key=key, validator=validator)
     for result in xoring:
         yield result
 
 
-@comprehension()
+@comprehension(chained=True)
 async def axor(self, *, key, validator):
     """
     Applies an xor to each result of any underlying async `Comprende`
@@ -98,12 +105,12 @@ async def axor(self, *, key, validator):
     underlying ``self`` async generator needs to produce 256-byte
     integers to be xor'd on each iteration.
     """
-    xoring = OneTimePad.axor.root(self, key=key, validator=validator)
+    xoring = Chunky2048.axor.root(self, key=key, validator=validator)
     async for result in xoring:
         yield result
 
 
-@comprehension()
+@comprehension(chained=True)
 def xor(self, *, key, validator):
     """
     Applies an xor to each result of any underlying sync `Comprende`
@@ -111,12 +118,12 @@ def xor(self, *, key, validator):
     underlying ``self`` sync generator needs to produce 256-byte
     integers to be xor'd on each iteration.
     """
-    xoring = OneTimePad.xor.root(self, key=key, validator=validator)
+    xoring = Chunky2048.xor.root(self, key=key, validator=validator)
     for result in xoring:
         yield result
 
 
-@comprehension()
+@comprehension(chained=True)
 async def apasscrypt(self, *, kb=1024, cpu=3, hardness=1024):
     """
     Applies the `passcrypt` algorithm on a pseudo-randomly generated
@@ -124,15 +131,15 @@ async def apasscrypt(self, *, kb=1024, cpu=3, hardness=1024):
     async generator. Each iteration a new salt is produced & is yield
     along with the result of the `passcrypt` operation.
     """
-    async for password in self:
+    got = None
+    passcrypt = Passcrypt(kb=kb, cpu=cpu, hardness=hardness)
+    while True:
         salt = await agenerate_salt()
-        result = await _apasscrypt(
-            password, salt, kb=kb, cpu=cpu, hardness=hardness
-        )
-        yield salt, result
+        result = await passcrypt.anew(await self.asend(got), salt)
+        got = yield salt, result
 
 
-@comprehension()
+@comprehension(chained=True)
 def passcrypt(self, *, kb=1024, cpu=3, hardness=1024):
     """
     Applies the `passcrypt` algorithm on a pseudo-randomly generated
@@ -140,66 +147,79 @@ def passcrypt(self, *, kb=1024, cpu=3, hardness=1024):
     sync generator. Each iteration a new salt is produced & is yield
     along with the result of the `passcrypt` operation.
     """
-    for password in self:
-        salt = generate_salt()
-        result = _passcrypt(
-            password, salt, kb=kb, cpu=cpu, hardness=hardness
-        )
-        yield salt, result
+    got = None
+    _passcrypt = Passcrypt(kb=kb, cpu=cpu, hardness=hardness)
+    try:
+        while True:
+            salt = generate_salt()
+            result = _passcrypt.new(self.send(got), salt)
+            got = yield salt, result
+    except StopIteration:
+        pass
 
 
-@comprehension()
+@comprehension(chained=True)
 async def asum_passcrypt(self, salt, *, kb=1024, cpu=3, hardness=1024):
     """
     Cumulatively applies the ``apasscrypt`` algorithm to each value
     that's yielded from the underlying Comprende async generator with
     the previously processed result before yielding the current result.
     """
+    got = None
+    passcrypt = Passcrypt(kb=kb, cpu=cpu, hardness=hardness)
     summary = await asha_512(salt, kb, cpu, hardness)
-    async for password in self:
-        pre_key = await asha_512(summary, salt, password)
-        summary = await _apasscrypt(
-            pre_key, summary, kb=kb, cpu=cpu, hardness=hardness
-        )
-        yield summary
+    while True:
+        pre_key = await asha_512(salt, summary, await self.asend(got))
+        summary = await passcrypt.anew(pre_key, summary)
+        got = yield summary
 
 
-@comprehension()
+@comprehension(chained=True)
 def sum_passcrypt(self, salt, *, kb=1024, cpu=3, hardness=1024):
     """
     Cumulatively applies the ``passcrypt`` algorithm to each value
     that's yielded from the underlying Comprende sync generator with
     the previously processed result before yielding the current result.
     """
+    got = None
+    passcrypt = Passcrypt(kb=kb, cpu=cpu, hardness=hardness)
     summary = sha_512(salt, kb, cpu, hardness)
-    for password in self:
-        pre_key = sha_512(summary, salt, password)
-        summary = _passcrypt(
-            pre_key, summary, kb=kb, cpu=cpu, hardness=hardness
-        )
-        yield summary
+    try:
+        while True:
+            pre_key = sha_512(salt, summary, self.send(got))
+            summary = passcrypt.new(pre_key, summary)
+            got = yield summary
+    except StopIteration:
+        pass
 
 
-@comprehension()
+@comprehension(chained=True)
 async def arandom_sleep(self, span=1):
     """
     Applies a random sleep before each yielded value from the underlying
     ``Comprende`` async generator.
     """
-    async for result in self:
+    got = None
+    asend = self.asend
+    while True:
         await _arandom_sleep(span)
-        yield result
+        got = yield await asend(got)
 
 
-@comprehension()
+@comprehension(chained=True)
 def random_sleep(self, span=1):
     """
     Applies a random sleep before each yielded value from the underlying
     ``Comprende`` sync generator.
     """
-    for result in self:
-        _random_sleep(span)
-        yield result
+    got = None
+    send = self.send
+    try:
+        while True:
+            _random_sleep(span)
+            got = yield send(got)
+    except StopIteration:
+        pass
 
 
 def insert_debuggers():
@@ -249,10 +269,10 @@ def insert_bytes_cipher_methods():
     Copies the addons over into the ``Comprende`` class.
     """
     addons = {
-        OneTimePad._bytes_encipher,
-        OneTimePad._bytes_decipher,
-        OneTimePad._abytes_encipher,
-        OneTimePad._abytes_decipher,
+        Chunky2048._bytes_encipher,
+        Chunky2048._bytes_decipher,
+        Chunky2048._abytes_encipher,
+        Chunky2048._abytes_decipher,
     }
     for addon in addons:
         name = addon.__name__[1:]
@@ -265,10 +285,10 @@ def insert_stream_cipher_methods():
     Copies the addons over into the ``Comprende`` class.
     """
     addons = {
-        OneTimePad._ascii_encipher,
-        OneTimePad._ascii_decipher,
-        OneTimePad._aascii_encipher,
-        OneTimePad._aascii_decipher,
+        Chunky2048._ascii_encipher,
+        Chunky2048._ascii_decipher,
+        Chunky2048._aascii_encipher,
+        Chunky2048._aascii_decipher,
     }
     for addon in addons:
         name = addon.__name__[1:]
@@ -281,10 +301,10 @@ def insert_hashmap_cipher_methods():
     Copies the addons over into the ``Comprende`` class.
     """
     addons = {
-        OneTimePad._map_encipher,
-        OneTimePad._map_decipher,
-        OneTimePad._amap_encipher,
-        OneTimePad._amap_decipher,
+        Chunky2048._map_encipher,
+        Chunky2048._map_decipher,
+        Chunky2048._amap_encipher,
+        Chunky2048._amap_decipher,
     }
     for addon in addons:
         setattr(Comprende, addon.__name__[1:], addon)
@@ -293,10 +313,10 @@ def insert_hashmap_cipher_methods():
 
 def insert_stateful_key_generator_objects():
     """
-    Copies the addons over into the ``OneTimePad`` class.
+    Copies the addons over into the ``Chunky2048`` class.
     """
 
-    def __init__(self, key=None):
+    def __init__(self, key=None, *, automate_key_use=True):
         """
         Creates an object which manages a main encryption key for use in
         a set of the package's static functions & generators. This
@@ -304,31 +324,25 @@ def insert_stateful_key_generator_objects():
         HMAC creation/validation by automatically passing in the key as
         a keyword argument.
         """
-        insert_keyrings(self, key)
-        self._key = self.keyring.key
-        self.hmac = self.keyring.hmac
-        self.ahmac = self.akeyring.ahmac
-        self.test_hmac = self.keyring.test_hmac
-        self.atest_hmac = self.akeyring.atest_hmac
-        self.passcrypt = self.keyring.passcrypt
-        self.apasscrypt = self.akeyring.apasscrypt
-        self.time_safe_equality = self.keyring.time_safe_equality
-        self.atime_safe_equality = self.akeyring.atime_safe_equality
-        for method in self.instance_methods:
-            convert_static_method_to_member(
-                self, method.__name__, method, key=self.key,
-            )
+        insert_keygens(self, key, automate_key_use=automate_key_use)
+        self._key = self.keygen.key
+        self.hmac = self.keygen.hmac
+        self.ahmac = self.akeygen.ahmac
+        self.test_hmac = self.keygen.test_hmac
+        self.atest_hmac = self.akeygen.atest_hmac
+        self.passcrypt = self.keygen.passcrypt
+        self.apasscrypt = self.akeygen.apasscrypt
+        self.time_safe_equality = self.keygen.time_safe_equality
+        self.atime_safe_equality = self.akeygen.atime_safe_equality
+        if automate_key_use:
+            for method in self.instance_methods:
+                convert_static_method_to_member(
+                    self, method.__name__, method, key=self.key
+                )
 
-    OneTimePad.__init__ = __init__
-    OneTimePad.Keys = Keys
-    OneTimePad.AsyncKeys = AsyncKeys
-    pad = OneTimePad(csprng())
-    validator.hmac = pad.hmac
-    validator.ahmac = pad.ahmac
-    validator.test_hmac = pad.test_hmac
-    validator.atest_hmac = pad.atest_hmac
-    validator.time_safe_equality = pad.time_safe_equality
-    validator.atime_safe_equality = pad.atime_safe_equality
+    Chunky2048.__init__ = __init__
+    Chunky2048.Keys = Keys
+    Chunky2048.AsyncKeys = AsyncKeys
 
 
 def add_protocols_to_collections():
@@ -339,6 +353,35 @@ def add_protocols_to_collections():
     X25519.protocols.Ropake = Ropake
 
 
+def insert_padding_methods():
+    """
+    Gives the `Padding` class access to methods defined in higher level
+    modules.
+    """
+    global pad_plaintext
+    global apad_plaintext
+    global depad_plaintext
+    global adepad_plaintext
+
+    Padding.derive_key = Chunky2048.padding_key
+    Padding.aderive_key = Chunky2048.apadding_key
+    pad_plaintext, apad_plaintext, depad_plaintext, adepad_plaintext = (
+        Padding.pad_plaintext,
+        Padding.apad_plaintext,
+        Padding.depad_plaintext,
+        Padding.adepad_plaintext,
+    )
+    addons = {
+        Padding._pad_plaintext,
+        Padding._depad_plaintext,
+        Padding._apad_plaintext,
+        Padding._adepad_plaintext,
+    }
+    for addon in addons:
+        setattr(Comprende, addon.__name__[1:], addon)
+        Comprende.lazy_generators.add(addon.__name__[1:])
+
+
 insert_debuggers()
 insert_xor_methods()
 insert_passcrypt_methods()
@@ -348,4 +391,5 @@ insert_stream_cipher_methods()
 insert_hashmap_cipher_methods()
 insert_stateful_key_generator_objects()
 add_protocols_to_collections()
+insert_padding_methods()
 
