@@ -2,7 +2,7 @@
 # crypto and anonymity library.
 #
 # Licensed under the AGPLv3: https://www.gnu.org/licenses/agpl-3.0.html
-# Copyright © 2019-2021 Gonzo Investigatory Journalism Agency, LLC
+# Copyright © 2019-2021 Gonzo Investigative Journalism Agency, LLC
 #            <gonzo.development@protonmail.ch>
 #           © 2019-2021 Richard Machado <rmlibre@riseup.net>
 # All rights reserved.
@@ -12,9 +12,9 @@
 __all__ = ["debuggers", "DebugControl"]
 
 
-__doc__ = """
-A collection of tools for helping with profiling and debugging.
-"""
+__doc__ = (
+    "A collection of tools for helping with profiling and debugging."
+)
 
 
 import inspect
